@@ -11,6 +11,7 @@ from .state import Constants as ingl_constants
 from rich import print
 from solana.rpc.async_api import AsyncClient
 from .cli_state import CLI_VERSION
+import os
 uasyncclient = Uasyncclient(rpc_url.target_network)
 
 @click.group()
@@ -18,17 +19,19 @@ uasyncclient = Uasyncclient(rpc_url.target_network)
 def entry():
     pass
 
-
 @click.group(name="mint")
 async def mint():
     pass
 
 async def mint_helper(keypair, log_level, class_enum):
-    log_level = int(log_level)
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
     mint_keypair = KeypairInput(keypair = Keypair())
     print("Mint_Id: ", mint_keypair.public_key)
     t_dets = await mint_nft(payer_keypair, mint_keypair, class_enum, client, log_level)
@@ -36,39 +39,39 @@ async def mint_helper(keypair, log_level, class_enum):
     await client.close()
 
 @click.command(name="Benitoite")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def Benitoite(keypair, log_level: int = 0):
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def Benitoite(keypair, log_level):
     await mint_helper(keypair, log_level, ClassEnum.enum.Benitoite())
 
 @click.command(name="Serendibite")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def Serendibite(keypair, log_level: int = 0):
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def Serendibite(keypair, log_level):
     await mint_helper(keypair, log_level, ClassEnum.enum.Serendibite())
 
 @click.command(name="Sapphire")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def Sapphire(keypair, log_level: int = 0):
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def Sapphire(keypair, log_level):
     await mint_helper(keypair, log_level, ClassEnum.enum.Sapphire())
 
 @click.command(name="Emerald")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def Emerald(keypair, log_level: int = 0):
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def Emerald(keypair, log_level):
     await mint_helper(keypair, log_level, ClassEnum.enum.Emerald())
 
 @click.command(name="Diamond")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def Diamond(keypair, log_level: int = 0):
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def Diamond(keypair, log_level):
     await mint_helper(keypair, log_level, ClassEnum.enum.Diamond())
 
 @click.command(name="Ruby")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def Ruby(keypair, log_level: int = 0):
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def Ruby(keypair, log_level):
     await mint_helper(keypair, log_level, ClassEnum.enum.Ruby())
 
 mint.add_command(Benitoite)
@@ -83,35 +86,83 @@ mint.add_command(Ruby)
 Create a proposal for the selection of the next validator whose vote account will be created.
 '''
 @click.command(name="create_validator_proposal")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def create_val_proposal(keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def create_val_proposal(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     gem_info = await client.get_account_info(global_gem_pubkey)
     numeration = GlobalGems.parse(gem_info.value.data).proposal_numeration
     t_dets = await create_validator_proposal(payer_keypair, numeration, client, log_level)
     print(t_dets)
     await client.close()
 
+@click.group(name="config")
+async def config():
+    pass
+
+@click.command(name = "set")
+@click.option("--program_id", "-p")
+@click.option("--url", "-u")
+@click.option("--keypair", "-k")
+def set(program_id, url, keypair):
+    assert program_id or url or keypair, "No options specified. Use --help for more information."
+    if program_id:
+        try:
+            program_pubkey = parse_pubkey_input(program_id)
+        except Exception as e:
+            print("Invalid Public Key provided.")
+            return
+        set_program_id(program_pubkey.public_key.__str__())
+        print("Program ID set to: ", program_pubkey.public_key)
+    if url:
+        if url.lower() == "mainnet" or url.lower() == "testnet" or url.lower() == "devnet":
+            url = rpc_url.get_network_url(url)
+        set_network(url)
+        print("Network set to: ", url)
+    if keypair:
+        set_keypair_path(keypair)
+        print("Keypair set to: ", keypair)
+    if not program_id and not url and not keypair:
+        print("No options specified. Use --help for more information.")
+        return
+    print("Config set successfully.")
+
+@click.command(name = "get")
+def get():
+    print("\nProgram ID: ", get_program_id())
+    print("Network: ", get_network())
+    print("Keypair: ", get_keypair_path())
+    print("\nConfig retrieved successfully.")
+
+config.add_command(set)
+config.add_command(get)
+
 '''
 Now Deprecated, do not use. Use create_vote_account instead.
 '''
 @click.command(name="finalize_proposal")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def finalize_validator_proposal(keypair, log_level: int = 0):
-    # log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def finalize_validator_proposal(keypair, log_level):
+
     # client = AsyncClient(rpc_url.target_network)
     # client_state = await client.is_connected()
     # print("Client is connected" if client_state else "Client is Disconnected")
-    # payer_keypair = parse_keypair_input(f"./{keypair}")
+    # try:
+    #     payer_keypair = parse_keypair_input(keypair)
+    # except Exception as e:
+    #     print("Invalid Keypair Input. ")
+    #     return
 
-    # global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    # global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     # gem_info = await client.get_account_info(global_gem_pubkey)
     # global_gems = GlobalGems.parse(gem_info.value.data)
     # numeration = global_gems.proposal_numeration
@@ -122,30 +173,45 @@ async def finalize_validator_proposal(keypair, log_level: int = 0):
 
 @click.command(name="allocate_gem")
 @click.argument('mint_id')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_allocate_gem(mint_id, keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--can_auto_delegate', '-a', default = True, type=bool)
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_allocate_gem(mint_id, keypair, can_auto_delegate, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    mint_pubkey = parse_pubkey_input(mint_id)
-    t_dets = await allocate_sol(payer_keypair, mint_pubkey, client, log_level)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        mint_pubkey = parse_pubkey_input(mint_id)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
+    t_dets = await allocate_sol(payer_keypair, mint_pubkey, can_auto_delegate, client, log_level)
     print(t_dets)
     await client.close()
 
 @click.command(name="deallocate_gem")
 @click.argument('mint_id')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_deallocate_gem(mint_id, keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_deallocate_gem(mint_id, keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    mint_pubkey = parse_pubkey_input(mint_id)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        mint_pubkey = parse_pubkey_input(mint_id)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
     t_dets = await deallocate_sol(payer_keypair, mint_pubkey, client, log_level)
     print(t_dets)
     await client.close()
@@ -153,74 +219,95 @@ async def process_deallocate_gem(mint_id, keypair, log_level: int = 0):
 
 
 @click.command(name='register_validator')
-@click.option('--keypair', default = 'keypair.json')
 @click.argument('validator_keypair')
-@click.option('--log_level', default = '2')
-async def reg_validator(validator_keypair, keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--log_level', '-l', default = '2', type=int)
+async def reg_validator(validator_keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    validator_keypair = parse_keypair_input(f"{validator_keypair}")
+    try:
+        validator_keypair = parse_keypair_input(f"{validator_keypair}")
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
     print("Validator Key: ", validator_keypair.public_key)
-    t_dets = await register_validator_id(payer_keypair, PubkeyInput(pubkey = validator_keypair.public_key), client, log_level)
+    t_dets = await register_validator_id(validator_keypair, client, log_level)
     print(t_dets)
     await client.close()
 
 @click.command(name="init_rebalance")
 @click.argument('vote_key')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def initialize_rebalancing(vote_key, keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def initialize_rebalancing(vote_key, keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    vote_account_id = parse_pubkey_input(vote_key)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        vote_account_id = parse_pubkey_input(vote_key)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
     t_dets = await init_rebalance(payer_keypair, vote_account_id, client, log_level)
     print(t_dets)
     await client.close()
 
 @click.command(name="finalize_rebalance")
 @click.argument('vote_key')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def finalize_rebalancing(vote_key, keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def finalize_rebalancing(vote_key, keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    vote_account_id = parse_pubkey_input(vote_key)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        vote_account_id = parse_pubkey_input(vote_key)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
     t_dets = await finalize_rebalance(payer_keypair, vote_account_id, client, log_level)
     print(t_dets)
     await client.close()
 
 @click.command(name="init")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def ingl(keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def ingl(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
     t_dets = await ingl_init(payer_keypair, client, log_level)
     print(t_dets)
     await client.close()
 
 @click.command(name="close_proposal")
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def close_val_proposal(keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def close_val_proposal(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     gem_info = await client.get_account_info(global_gem_pubkey)
     numeration = GlobalGems.parse(gem_info.value.data).proposal_numeration
     t_dets = await close_proposal(payer_keypair, numeration-1, client, log_level)
@@ -228,30 +315,40 @@ async def close_val_proposal(keypair, log_level: int = 0):
     await client.close()
 
 @click.command(name="process_rewards")
-@click.option('--keypair', default = 'keypair.json')
+@click.option('--keypair', '-k', default = get_keypair_path())
 @click.argument('vote_key')
-@click.option('--log_level', default = '2')
-async def process_vote_account_rewards(keypair, vote_key, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_vote_account_rewards(keypair, vote_key, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}") 
-    vote_account_id = parse_pubkey_input(vote_key)
+    try:
+        payer_keypair = parse_keypair_input(keypair) 
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        vote_account_id = parse_pubkey_input(vote_key)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
     t_dets = await process_rewards(payer_keypair, vote_account_id, client, log_level)
     print(t_dets)
     await client.close()
 
 @click.command(name='create_vote_account')
-@click.option('--val_keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_create_vote_account(val_keypair, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--val_keypair', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_create_vote_account(val_keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{val_keypair}")
-    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    try:
+        payer_keypair = parse_keypair_input(f"./{val_keypair}")
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     gem_info = await client.get_account_info(global_gem_pubkey)
     global_gems = GlobalGems.parse(gem_info.value.data)
     numeration = global_gems.proposal_numeration
@@ -263,19 +360,25 @@ async def process_create_vote_account(val_keypair, log_level: int = 0):
     await client.close()
 
 @click.command(name='vote_validator')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
 @click.argument('mint_id')
-@click.argument("validator_index")
-async def process_vote_validator_proposal(keypair, mint_id, validator_index, log_level: int = 0):
-    validator_index = int(validator_index)
-    log_level = int(log_level)
+@click.argument("validator_index", type=int)
+async def process_vote_validator_proposal(keypair, mint_id, validator_index, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    mint_pubkey = parse_pubkey_input(mint_id)
-    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        mint_pubkey = parse_pubkey_input(mint_id)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     gem_info = await client.get_account_info(global_gem_pubkey)
     global_gems = GlobalGems.parse(gem_info.value.data)
     numeration = global_gems.proposal_numeration
@@ -289,32 +392,50 @@ async def process_vote_validator_proposal(keypair, mint_id, validator_index, log
 @click.command(name='delegate_gem')
 @click.argument('mint_id')
 @click.argument('vote_account')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_delegate_gem(keypair, mint_id, vote_account, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_delegate_gem(keypair, mint_id, vote_account, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    mint_pubkey = parse_pubkey_input(mint_id)
-    vote_account_pubkey = parse_pubkey_input(vote_account)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        mint_pubkey = parse_pubkey_input(mint_id)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
+    try:
+        vote_account_pubkey = parse_pubkey_input(vote_account)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
     t_dets = await delegate_nft(payer_keypair, mint_pubkey, vote_account_pubkey, client, log_level)
     print(t_dets)
     await client.close()
 
-@click.command('undelegate_gem')
+@click.command(name = 'undelegate_gem')
 @click.argument('mint_id')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_undelegate_gem(keypair, mint_id, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_undelegate_gem(keypair, mint_id, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    mint_pubkey = parse_pubkey_input(mint_id)
-    gem_account_pubkey, _gem_account_bump = PublicKey.find_program_address([bytes(ingl_constants.GEM_ACCOUNT_CONST, 'UTF-8'), bytes(mint_pubkey.public_key)], ingl_constants.INGL_PROGRAM_ID)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        mint_pubkey = parse_pubkey_input(mint_id)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
+    gem_account_pubkey, _gem_account_bump = PublicKey.find_program_address([bytes(ingl_constants.GEM_ACCOUNT_CONST, 'UTF-8'), bytes(mint_pubkey.public_key)], get_program_id())
     gem_account = await client.get_account_info(gem_account_pubkey)
     gem_account_data = gem_account.value.data
     funds_location_data = gem_account_data[20:52]
@@ -327,36 +448,46 @@ async def process_undelegate_gem(keypair, mint_id, log_level: int = 0):
 @click.command(name='create_upgrade_proposal')
 @click.argument('buffer')
 @click.argument('code_link')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_create_upgrade_proposal(keypair, buffer, code_link, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_create_upgrade_proposal(keypair, buffer, code_link, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
-    buffer_address = parse_pubkey_input(buffer)
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
+    try:
+        buffer_address = parse_pubkey_input(buffer)
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
 
     print(f"Code_Link: [link={code_link}]{code_link}[/link]")
-    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     gem_info = await client.get_account_info(global_gem_pubkey)
     numeration = GlobalGems.parse(gem_info.value.data).upgrade_proposal_numeration
     t_dets = await create_program_upgrade_proposal(payer_keypair, buffer_address, numeration, code_link, client, log_level)
     print(t_dets)
     await client.close()
 
-@click.command(name='create_upgrade_proposal')
+@click.command(name='vote_upgrade_proposal')
 @click.argument('vote')
 @click.argument('proposal')
 @click.argument('vote_account')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_vote_upgrade_proposal(keypair, vote, proposal, vote_account, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_vote_upgrade_proposal(keypair, vote, proposal, vote_account, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
     vote = parse_vote(vote)
     upgrade_proposal_pubkey, upgrade_proposal_numeration = parse_proposal(proposal)
     vote_account_key, vote_account_numeration = parse_proposal(vote_account)
@@ -365,16 +496,19 @@ async def process_vote_upgrade_proposal(keypair, vote, proposal, vote_account, l
     print(t_dets)
     await client.close()
 
-@click.command(name='create_upgrade_proposal')
+@click.command(name='finalize_upgrade_proposal')
 @click.argument('proposal')
-@click.option('--keypair', default = 'keypair.json')
-@click.option('--log_level', default = '2')
-async def process_finalize_upgrade_proposal(keypair, proposal, log_level: int = 0):
-    log_level = int(log_level)
+@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--log_level', '-l', default = '2', type=int)
+async def process_finalize_upgrade_proposal(keypair, proposal, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
     print("Client is connected" if client_state else "Client is Disconnected")
-    payer_keypair = parse_keypair_input(f"./{keypair}")
+    try:
+        payer_keypair = parse_keypair_input(keypair)
+    except Exception as e:
+        print("Invalid Keypair Input. ")
+        return
     upgrade_proposal_pubkey, upgrade_proposal_numeration = parse_proposal(proposal)
     print(f"Proposal_Account {upgrade_proposal_pubkey}");
     t_dets = await finalize_program_upgrade_proposal(payer_keypair, upgrade_proposal_pubkey, upgrade_proposal_numeration, client, log_level)
@@ -383,29 +517,33 @@ async def process_finalize_upgrade_proposal(keypair, proposal, log_level: int =
 
 
 @click.command(name="get_vote_pubkey")
-@click.option('--numeration', '-n', default=None)
+@click.option('--numeration', '-n', default=None, type=int)
 def get_vote_pubkey(numeration):
     if numeration:
-        expected_vote_pubkey, _expected_vote_pubkey_nonce = PublicKey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8"), (int(numeration)-1).to_bytes(4,"big")], ingl_constants.INGL_PROGRAM_ID)
+        expected_vote_pubkey, _expected_vote_pubkey_nonce = PublicKey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8"), (numeration-1).to_bytes(4,"big")], get_program_id())
     else:
-        global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+        global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
         numeration = GlobalGems.parse(uasyncclient.get_account_info(global_gem_pubkey).value.data).proposal_numeration - 1 
         if numeration < 0:
             print("Please precise the Proposal numeration using the '-n' command or the  '--numeration' command ")
             return
-        expected_vote_pubkey, _expected_vote_pubkey_nonce = PublicKey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8"), (numeration).to_bytes(4,"big")], ingl_constants.INGL_PROGRAM_ID)
+        expected_vote_pubkey, _expected_vote_pubkey_nonce = PublicKey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8"), (numeration).to_bytes(4,"big")], get_program_id())
         print("Vote account for Proposal No: ", numeration, ". You can precise the specific proposal with the '-n' option")
     print("Vote Pubkey: ", expected_vote_pubkey)
 
 @click.command(name="find_vote_key")
 @click.argument('val_pubkey')
 def find_vote_key(val_pubkey):
-    val_pubkey = parse_pubkey_input(val_pubkey) 
-    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
+    try:
+        val_pubkey = parse_pubkey_input(val_pubkey) 
+    except Exception as e:
+        print("Invalid Public Key provided.")
+        return
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], get_program_id())
     numeration = GlobalGems.parse(uasyncclient.get_account_info(global_gem_pubkey).value.data).proposal_numeration
     for i in range(numeration):
-        expected_vote_pubkey, _expected_vote_pubkey_nonce = PublicKey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8"), (i).to_bytes(4,"big")], ingl_constants.INGL_PROGRAM_ID)
-        expected_vote_data_pubkey, _expected_vote_data_bump = PublicKey.find_program_address([bytes(ingl_constants.VOTE_DATA_ACCOUNT_KEY, 'UTF-8'), bytes(expected_vote_pubkey)], ingl_constants.INGL_PROGRAM_ID)
+        expected_vote_pubkey, _expected_vote_pubkey_nonce = PublicKey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8"), (i).to_bytes(4,"big")], get_program_id())
+        expected_vote_data_pubkey, _expected_vote_data_bump = PublicKey.find_program_address([bytes(ingl_constants.VOTE_DATA_ACCOUNT_KEY, 'UTF-8'), bytes(expected_vote_pubkey)], get_program_id())
         validator_id = PublicKey(InglVoteAccountData.parse(uasyncclient.get_account_info(expected_vote_data_pubkey).value.data).validator_id)
         if validator_id == val_pubkey.public_key:
             print("Vote_Pubkey: ", expected_vote_pubkey)
@@ -435,5 +573,6 @@ entry.add_command(process_undelegate_gem)
 entry.add_command(process_create_upgrade_proposal)
 entry.add_command(process_vote_upgrade_proposal)
 entry.add_command(process_finalize_upgrade_proposal)
+entry.add_command(config)
 if __name__ == '__main__':
     entry()
\ No newline at end of file
