@@ -9,6 +9,7 @@ from solana.rpc import types
 from solana.rpc.async_api import AsyncClient
 from solana.transaction import Transaction
 from solders.rpc.responses import SendTransactionResp
+import os
 
 
 class Constants:
@@ -17,7 +18,7 @@ class Constants:
     INGL_MINTING_POOL_KEY = "minting_pool"
     INGL_TEAM_ACCOUNT = "ingl_team_key"
     COLLECTION_HOLDER_KEY = "collection_holder"
-    INGL_PROGRAM_ID = PublicKey("6Rw7Jif6Mx9BVdzk9rwZmuMQ9Edjygj1FSe2VJ7EaghA")
+    INGL_CONFIG_ACCOUNT_KEY = "config_account"
     STAKE_PROGRAM_ID = PublicKey("Stake11111111111111111111111111111111111111")
     BPF_LOADER_KEY = PublicKey("BPFLoaderUpgradeab1e11111111111111111111111")
     GLOBAL_GEM_KEY = "global_gem_account"
@@ -64,6 +65,30 @@ ClassEnum = Enum(
     enum_name = "ClassEnum",
 )
 
+Rarity = Enum(
+    "Common",
+    "Uncommon",
+    "Rare",
+    "Exalted",
+    "Mythic",
+
+    enum_name = "Rarity",
+)
+
+def int_from_class_enum(class_enum: ClassEnum.enum) -> int:
+    if class_enum == ClassEnum.enum.Ruby():
+        return 0
+    elif class_enum == ClassEnum.enum.Diamond():
+        return 1
+    elif class_enum == ClassEnum.enum.Sapphire():
+        return 2
+    elif class_enum == ClassEnum.enum.Emerald():
+        return 3
+    elif class_enum == ClassEnum.enum.Serendibite():
+        return 4
+    elif class_enum == ClassEnum.enum.Benitoite():
+        return 5
+
 def keypair_from_json(filepath):
     keypair = Keypair.from_secret_key(json.load(open(filepath)))
     return keypair
@@ -150,6 +175,16 @@ class rpc_url:
         else:
             return ""
 
+    def get_network_url(network):
+        if network == "devnet":
+            return rpc_url.DEVNET
+        elif network == "testnet":
+            return rpc_url.TESTNET
+        elif network == "mainnet":
+            return rpc_url.MAINNET
+        else:
+            raise Exception("Invalid network")
+
 
 
 class KeypairInput:
@@ -185,9 +220,14 @@ def parse_pubkey_input(str_input: String) -> PubkeyInput:
             return pubkey
         except Exception as e:
             if 'invalid public key input:' in str(e):
-                t_keypair = keypair_from_json(str_input)
-                return PubkeyInput(keypair=t_keypair, pubkey=t_keypair.public_key)
+                try:
+                    t_keypair = keypair_from_json(str_input)
+                    return PubkeyInput(keypair=t_keypair, pubkey=t_keypair.public_key)
+                except Exception as new_e:
+                    print("invalid public key input")
+                    raise new_e
             else:
+                print("invalid public key input")
                 raise e
 
 async def sign_and_send_tx(tx: Transaction, client: AsyncClient, *args) -> SendTransactionResp:
@@ -236,12 +276,12 @@ def parse_upgrade_proposal_id(proposal_pubkey: Optional[PublicKey], numeration:
     proposal_account_pubkey = PublicKey(1)
     proposal_numeration = 0
     if numeration:
-        proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.UPGRADE_PROPOSAL_KEY, 'UTF-8'), (numeration).to_bytes(4,"big")], Constants.INGL_PROGRAM_ID)[0]
+        proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.UPGRADE_PROPOSAL_KEY, 'UTF-8'), (numeration).to_bytes(4,"big")], get_program_id())[0]
         proposal_numeration = numeration
         return proposal_account_pubkey, proposal_numeration
     else:
        while cnt > 0:
-            proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.UPGRADE_PROPOSAL_KEY, 'UTF-8'), (cnt-1).to_bytes(4,"big")], Constants.INGL_PROGRAM_ID)[0]
+            proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.UPGRADE_PROPOSAL_KEY, 'UTF-8'), (cnt-1).to_bytes(4,"big")], get_program_id())[0]
             if proposal_account_pubkey == proposal_pubkey.public_key:
                 proposal_numeration = cnt
                 break
@@ -255,12 +295,12 @@ def parse_validator_proposal_id(proposal_pubkey: Optional[PublicKey], numeration
     proposal_account_pubkey = PublicKey(1)
     proposal_numeration = 0
     if numeration:
-        proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.VOTE_ACCOUNT_KEY, 'UTF-8'), (numeration).to_bytes(4,"big")], Constants.INGL_PROGRAM_ID)[0]
+        proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.VOTE_ACCOUNT_KEY, 'UTF-8'), (numeration).to_bytes(4,"big")], get_program_id())[0]
         proposal_numeration = numeration
         return proposal_account_pubkey, proposal_numeration
     else:
        while cnt > 0:
-            proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.VOTE_ACCOUNT_KEY, 'UTF-8'), (cnt-1).to_bytes(4,"big")], Constants.INGL_PROGRAM_ID)[0]
+            proposal_account_pubkey = PublicKey.find_program_address([bytes(Constants.VOTE_ACCOUNT_KEY, 'UTF-8'), (cnt-1).to_bytes(4,"big")], get_program_id())[0]
             if proposal_account_pubkey == proposal_pubkey.public_key:
                 proposal_numeration = cnt
                 break
@@ -268,4 +308,65 @@ def parse_validator_proposal_id(proposal_pubkey: Optional[PublicKey], numeration
                 cnt -=1
     if proposal_account_pubkey != proposal_pubkey.public_key:
         raise Exception("Proposal not found")
-    return proposal_account_pubkey, proposal_numeration
\ No newline at end of file
+    return proposal_account_pubkey, proposal_numeration
+
+
+def set_config(key: str, value: str):
+    file_dir = f"{os.path.expanduser('~')}/.config/solana/ingl/"
+    os.makedirs(file_dir, exist_ok=True)
+    file_dir = file_dir + "config.json"
+    try:
+        f = open(file_dir, 'r')
+        config = json.load(f)
+        f.close()
+    except:
+        config = {}
+    config[key] = value
+    with open(file_dir, 'w') as f:
+        json.dump(config, f)
+
+def get_config(key: str) -> str:
+    file_dir = f"{os.path.expanduser('~')}/.config/solana/ingl/"
+    os.makedirs(file_dir, exist_ok=True)
+    file_dir = file_dir + "config.json"
+    try:
+        f = open(file_dir, 'r')
+        config = json.load(f)
+        f.close()
+        if key in config:
+            return config[key]
+        else:
+            return ""
+    except:
+        return ""
+
+
+def get_program_id() -> PublicKey:
+    program_id_str = get_config('program_id')
+    try:
+        return PublicKey(program_id_str)
+    except:
+        return PublicKey("AtTUCuWgLnPBiuVmoyXJJ1PbTXtQD81U569qAocvWRTQ")
+
+def set_program_id(program_id: str):
+    set_config('program_id', program_id)
+
+def get_network() -> str:
+    network = get_config('network')
+    if network == "":
+        return rpc_url.get_network_url('devnet')
+    else:
+        return network
+
+def set_network(network: str):
+    set_config('network', network)
+
+def get_keypair_path() -> str:
+    keypair_path = get_config('keypair_path')
+    if keypair_path == "":
+        return f"{os.path.expanduser('~')}/.config/solana/ingl/id.json"
+    else:
+        return keypair_path
+    
+def set_keypair_path(keypair_path: str):
+    set_config('keypair_path', keypair_path)
\ No newline at end of file
